"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const authentication = require("feathers-authentication-client");
const hooks = require("feathers-hooks");
const localstorage = require("feathers-localstorage");
const socketio = require("feathers-socketio/client");
const feathers = require("feathers/client");
const storage = require("localstorage-memory");
const prom_client_1 = require("prom-client");
class Gateway {
    constructor(client, url) {
        this.client = client;
        this.url = url;
        this.connected = false;
        this.gauges = {};
        this.registry = new prom_client_1.Registry();
        this.authenticating = false;
        this.authenticated = false;
        client.emit("debug", "Connecting to: " + url);
        this.socket = require("socket.io-client")(url);
        this.socket.on("disconnect", () => {
            if (!this.reconnectInterval) {
                this.reconnectInterval = setInterval(() => {
                    this.socket.connect();
                }, 5000);
            }
        });
        this.socket.on("connect", () => {
            this.connected = true;
            this.client.emit("gatewayConnect", this);
            this.client.emit("debug", "Connected to: " + url);
            this.authenticated ? this.configure() : this.authenticate();
            if (this.reconnectInterval) {
                clearInterval(this.reconnectInterval);
            }
        });
        this.socket.on("fetchMetrics", (time, ack) => {
            ack(this.registry.metrics());
        });
        this.socket.on("reconnect", () => {
            this.client.emit("debug", "Reconnected to: " + url);
        });
        this.socket.on("error", (e) => this.client.emit("error", "error", e));
        this.socket.on("connect_error", (e) => this.client.emit("error", "connection error", e));
        this.socket.on("reconnect_error", (e) => this.client.emit("error", "reconnection error", e));
        this.socket.on("connect_timeout", () => this.client.emit("error", "connection timeout"));
        this.storage = localstorage({ storage });
        if (client.config.gateway.events) {
            client.config.gateway.events.forEach((x) => {
                this.socket.on(x, (...args) => this.client.emit(x, ...args, this));
            });
        }
        this.app = feathers()
            .configure(socketio(this.socket))
            .configure(hooks())
            .configure(authentication(Object.assign({ service: "agent", entity: "agent", storage }, client.config.auth)));
    }
    get rest() {
        return this.configuration.rest;
    }
    authenticate() {
        return new Promise((resolve, reject) => {
            this.client.emit("debug", "Authenticating: " + this.url);
            if (this.authenticating || this.authenticated) {
                return;
            }
            this.authenticating = true;
            const auth = {
                strategy: "local",
                type: "local",
                name: "internal",
                key: this.client.config.auth.key,
            };
            this.app.authenticate(auth)
                .then((result) => {
                this.authenticating = false;
                this.client.emit("gatewayAuthenticated", this);
                this.client.emit("debug", "Authenticated: " + this.url);
                this.authToken = result.accessToken;
                this.configure(resolve);
            })
                .catch((error) => {
                this.authenticating = false;
                console.error(error);
                this.client.emit("error", error);
                reject(error);
            });
        });
    }
    addGauge(nick, help, labelNames = []) {
        const name = (this.client.config.promPrefix || "discordservers_prod_bot_") + nick;
        this.gauges[nick] = new prom_client_1.Gauge({ name, help, labelNames, registers: [this.registry] });
    }
    emit(event, ...args) {
        this.socket.emit(event, ...args);
    }
    on(event, callback) {
        this.socket.on(event, callback);
    }
    configure(callback = () => {
        }) {
        this.client.emit("debug", "Configuring: " + this.url);
        this.socket.emit("configure", this.client.config.gateway, (configuration) => {
            this.configuration = configuration;
            this.client.emit("gatewayConfigured", this);
            this.client.emit("debug", "Configured: " + this.url);
            callback();
        });
    }
}
exports.default = Gateway;
//# sourceMappingURL=Gateway.js.map